package com.jeeplus.test.roster.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jeeplus.centerservice.utils.RestTemplateService;
import com.jeeplus.core.excel.utils.EasyPoiUtil;
import com.jeeplus.core.query.QueryWrapperGenerator;
import com.jeeplus.security.util.SecurityUtils;
import com.jeeplus.sys.constant.enums.OfficeTypeEnum;
import com.jeeplus.sys.domain.*;
import com.jeeplus.sys.mapper.SysUserFailedLogMapper;
import com.jeeplus.sys.mapper.UserMapper;
import com.jeeplus.sys.service.CertService;
import com.jeeplus.sys.service.OfficeService;
import com.jeeplus.sys.service.SysUserFailedLogService;
import com.jeeplus.sys.service.UserService;
import com.jeeplus.sys.service.dto.CertDTO;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.service.mapstruct.CertWrapper;
import com.jeeplus.sys.utils.DictUtils;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.sys.utils.UserUtils;
import com.jeeplus.test.cw.common.CommonUtils;
import com.jeeplus.test.jobPosion.utils.BeanUtils;
import com.jeeplus.test.mould.service.SerialnumTplService;
import com.jeeplus.test.oss.service.OssService;
import com.jeeplus.test.roster.domain.*;
import com.jeeplus.test.roster.mapper.RosterBaseMapper;
import com.jeeplus.test.roster.service.dto.*;
import com.jeeplus.test.roster.service.mapstruct.*;
import com.jeeplus.center.enums.ServiceAliasEnum;
import com.sun.org.apache.xpath.internal.operations.Bool;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

import static com.jeeplus.test.jobPosion.utils.BeanUtils.objectCheckIsNull;

@Service
@Transactional(rollbackFor = Exception.class)
public class RosterBaseService extends ServiceImpl<RosterBaseMapper, RosterBase> {

    @Autowired
    private RosterBaseMapper rosterBaseMapper;

    @Autowired
    private RosterContractService rosterContractService;

    @Autowired
    private RosterBankCardsService rosterBankCardsService;

    @Autowired
    private RosterEducateService rosterEducateService;

    @Autowired
    private RosterFamilyService rosterFamilyService;

    @Autowired
    private RosterWorkService rosterWorkService;

    @Autowired
    private RosterFilesService rosterFilesService;

    @Autowired
    private RosterCertificateService rosterCertificateService;

    @Autowired
    private RosterLeadershipService rosterLeadershipService;

    @Autowired
    private OssService ossService;

    @Autowired
    private SerialnumTplService serialnumTplService;

    @Autowired
    private OfficeService officeService;

    @Autowired
    private UserService userService;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private RestTemplateService restTemplateService;

    @Autowired
    private SysUserFailedLogMapper sysUserFailedLogMapper;

    @Autowired
    private CertService certService;

    @Autowired
    private SysUserFailedLogService sysUserFailedLogService;

    public IPage<RosterBaseDTO> findList(Page<RosterBaseDTO> page, RosterBaseDTO rosterBaseDTO) throws Exception {
        RosterBase rosterBase = RosterBaseWrapper.INSTANCE.toEntity(rosterBaseDTO);
        QueryWrapper<RosterBase> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( rosterBase, RosterBase.class );
        queryWrapper.eq("rb.del_flag","0");
        if(ObjectUtil.isNotEmpty(rosterBaseDTO)){
            if(StringUtils.isNotBlank(rosterBaseDTO.getCompanyId())){
                // 根据公司层级筛选
                queryWrapper.eq("soc.id",rosterBaseDTO.getCompanyId());
            }else if(StringUtils.isNotBlank(rosterBaseDTO.getOfficeId())){
                // 根据部门层级筛选
                queryWrapper.eq("soo.id",rosterBaseDTO.getOfficeId());
            }
            if (StringUtils.isNotBlank(rosterBaseDTO.getOfficeSelect())) {
                //  根据部门筛选
                List<User> userList = userService.list(new LambdaQueryWrapper<User>().eq(User::getOfficeId, rosterBaseDTO.getOfficeSelect()));
                List<String> ids = userList.stream().distinct().map(User::getRosterId).collect(Collectors.toList());
                queryWrapper.in("rb.id",ids);
            }
            if (StringUtils.isNotBlank(rosterBaseDTO.getPostSelect())) {
                //  根据岗位筛选
                List<User> userByPost = userService.getUserIdByPost(rosterBaseDTO.getPostSelect());
                List<String> ids = userByPost.stream().distinct().map(User::getRosterId).collect(Collectors.toList());
                queryWrapper.in("rb.id",ids);
            }
            if (ObjectUtil.isNotEmpty(rosterBaseDTO.getDates())) {
                queryWrapper.between("rb.onboarding_date", rosterBaseDTO.getDates()[0], rosterBaseDTO.getDates()[1]);
            }
        }
        IPage<RosterBaseDTO> rosterBaseDTOIPage = rosterBaseMapper.findList(page, queryWrapper);
        if(CollectionUtil.isNotEmpty(rosterBaseDTOIPage.getRecords())){
            rosterBaseDTOIPage.getRecords().stream().forEach(item->{
//                头像获取到新的临时地址展示
//                if(StringUtils.isNotBlank(item.getHeadshot())){
//                    String fileTemporaryLookUrl = ossService.getFileTemporaryLookUrl(item.getHeadshot());
//                    item.setHeadshotLsUrl(fileTemporaryLookUrl);
//                }
                if (CollectionUtil.isNotEmpty(item.getPostList())) {
                    String postNames = item.getPostList().stream().filter(filter ->{
                        if(ObjectUtil.isNotEmpty(filter)) {
                            if (StringUtils.isNotBlank(filter.getName())) {
                                return true;
                            }
                        }
                        return false;
                    }).map(Post::getName).collect(Collectors.joining(","));
                    item.setPostNames(postNames);
                }
                if (CollectionUtil.isNotEmpty(item.getRoleList())) {
                    String roleNames = item.getRoleList().stream().filter(filter ->{
                        if(ObjectUtil.isNotEmpty(filter)) {
                            if (StringUtils.isNotBlank(filter.getName())) {
                                return true;
                            }
                        }
                        return false;
                    }).map(Role::getName).collect(Collectors.joining(","));
                    item.setRoleNames(roleNames);
                }
                // 查询是否有需要手动推送到子系统的数据
                Map<String,Object> checkResp = checkIsSyncFailed(item.getId());
                item.setPushFailed((Boolean)checkResp.get("success"));
            });
        }
        return rosterBaseDTOIPage;
    }

    public RosterBaseDTO queryById(String id){
        RosterBaseDTO rosterBaseDTO = rosterBaseMapper.queryById(id);
        if(ObjectUtil.isNotEmpty(rosterBaseDTO)){
            if(StringUtils.isNotBlank(rosterBaseDTO.getCertificatesFront())){
                String front = ossService.getFileTemporaryLookUrl(rosterBaseDTO.getCertificatesFront());
                rosterBaseDTO.setCertificatesFrontLsUrl(front);
            }
            if(StringUtils.isNotBlank(rosterBaseDTO.getHeadshot())){
                String headshot = ossService.getFileTemporaryLookUrl(rosterBaseDTO.getHeadshot());
                rosterBaseDTO.setHeadshotLsUrl(headshot);
            }
            if(StringUtils.isNotBlank(rosterBaseDTO.getCertificatesBack())){
                String back = ossService.getFileTemporaryLookUrl(rosterBaseDTO.getCertificatesBack());
                rosterBaseDTO.setCertificatesBackLsUrl(back);
            }
            if(CollectionUtil.isNotEmpty(rosterBaseDTO.getRosterBankCardsDTOList())){
                rosterBaseDTO.getRosterBankCardsDTOList().stream().forEach(item->{
                    if(StringUtils.isNotBlank(item.getPayrollCardPic())){
                        String cardPic = ossService.getFileTemporaryLookUrl(item.getPayrollCardPic());
                        item.setPayrollCardPicLsUrl(cardPic);
                    }
                });
            }
            if(CollectionUtil.isNotEmpty(rosterBaseDTO.getRosterEducateDTOList())){
                rosterBaseDTO.getRosterEducateDTOList().stream().forEach(item->{
                    if(StringUtils.isNotBlank(item.getEducationCertFirst())){
                        String certFirst = ossService.getFileTemporaryLookUrl(item.getEducationCertFirst());
                        item.setEducationCertFirstLsUrl(certFirst);
                    }
                    if(StringUtils.isNotBlank(item.getEducationCertFirst())){
                        String certSecond = ossService.getFileTemporaryLookUrl(item.getEducationCertSecond());
                        item.setEducationCertSecondLsUrl(certSecond);
                    }
                });
            }
            if(CollectionUtil.isNotEmpty(rosterBaseDTO.getRosterFamilyDTOList())){
                rosterBaseDTO.getRosterFamilyDTOList().stream().forEach(item->{
                    if(StringUtils.isNotBlank(item.getBirthCertificate())){
                        String birth = ossService.getFileTemporaryLookUrl(item.getBirthCertificate());
                        item.setBirthCertificateLsUrl(birth);
                    }
                });
            }
            // 查询花名册关联用户的信息
            User user = userService.getOne(new QueryWrapper<User>().lambda().eq(User::getRosterId, id));
            List<CertDTO> certDTOList = new ArrayList<>();
            if (ObjectUtil.isNotEmpty(user)) {
                // 获取用户资质
                List<Cert> certList = userService.getCertListByUserId(user.getId());  // 根据用户id获取资质信息
                if (CollectionUtil.isNotEmpty(certList)) {
                    certList.stream().forEach(c -> {
                        CertDTO certDTO = CertWrapper.INSTANCE.toDTO(c);
                        if (StringUtils.isNotBlank(certDTO.getFileUrl())) {  //  根据oss图片路径获取图片临时地址
                            String fileLsUrl = ossService.getFileTemporaryLookUrl(certDTO.getFileUrl());
                            certDTO.setFileLsUrl(fileLsUrl);
                        }
                        certDTOList.add(certDTO);
                    });
                }
                // 获取用户的公司
                if (StringUtils.isNotBlank(user.getCompanyId())) {
                    rosterBaseDTO.setCompanyId(user.getCompanyId());
                }
                // 获取用户角色
                List<String> userRole = userMapper.getUserRole(user.getId());
                if (CollectionUtil.isNotEmpty(userRole)) {
                    rosterBaseDTO.setRoleIdList(userRole);
                }
                // 获取用户岗位
                List<String> userPost = userMapper.getUserPost(user.getId());
                if (CollectionUtil.isNotEmpty(userPost)) {
                    rosterBaseDTO.setPostIdList(userPost);
                }
                // 获取用户的管理部门
                if (StringUtils.isNotBlank(user.getManageOfficeIds())) {
                    rosterBaseDTO.setManageOfficeIds(user.getManageOfficeIds());
                }
            }
            rosterBaseDTO.setCertDTOList(certDTOList);
        }
        return rosterBaseDTO;
    }

    public ResponseEntity saveRoster(RosterBaseDTO rosterBaseDTO) throws Exception{

        String oldOfficeId = "";

        if(ObjectUtil.isNotEmpty(rosterBaseDTO)){
            if(StringUtils.isNotBlank(rosterBaseDTO.getId())){
                RosterBase byId = this.getById(rosterBaseDTO.getId());
                oldOfficeId = StringUtils.isNotBlank(byId.getOfficeId()) ? byId.getOfficeId() : "";
                // 原数据的人员类型为员工，现在要修改成游客，
                // 原数据关联的用户删除掉
                // 删除原数据关联表的数据，删除与游客无关的数据
                if ("1".equals(byId.getPersonType()) && "2".equals(rosterBaseDTO.getPersonType())) {
                    // 原数据关联的用户删除掉
                    // 通过花名册id查询用户
                    List<User> ulist = userService.list(new QueryWrapper<User>().lambda().eq(User::getRosterId, rosterBaseDTO.getId()));
                    if (CollectionUtil.isNotEmpty(ulist)) {
                        ulist.stream().forEach(i -> {
                            // 通过花名册id删除用户
                            userService.removeById(i.getId());
                            UserDTO u = new UserDTO();
                            u.setId(i.getId());
                            // 删除用户关联的管理部门信息
                            userMapper.deleteUserManageOffice(u);  //  根据用户id删除用户的管理部门信息
                            // 删除用户关联的岗位信息
                            userMapper.deleteUserPost(i.getId());
                            // 删除用户关联的角色信息
                            userMapper.deleteUserRole(i.getId());
                        });
                    }
                    //  删除原数据关联表的数据，清空与游客无关的字段
                    rosterBaseDTO = this.deleteRosterWorker(rosterBaseDTO);
                }
                // 身份证号唯一验证（排除自己）
                if(StringUtils.isNotBlank(rosterBaseDTO.getIdCard())){
                    List<RosterBase> rosterBaseList = this.list(new QueryWrapper<RosterBase>().eq("id_card", rosterBaseDTO.getIdCard()).ne("id",rosterBaseDTO.getId()));
                    if(CollectionUtil.isNotEmpty(rosterBaseList)){
                        return ResponseEntity.badRequest().body("基础信息-身份证号 已存在，保存失败");
                    }
                }
                // 手机号唯一验证（排除自己）
                if(StringUtils.isNotBlank(rosterBaseDTO.getMobile())){
                    List<RosterBase> rosterBaseList = this.list(new QueryWrapper<RosterBase>().eq("mobile", rosterBaseDTO.getMobile()).ne("id",rosterBaseDTO.getId()));
                    if(CollectionUtil.isNotEmpty(rosterBaseList)){
                        return ResponseEntity.badRequest().body("基础信息-手机号码 已存在，保存失败");
                    }
                }
            }else{
                // 身份证号唯一验证
                if(StringUtils.isNotBlank(rosterBaseDTO.getIdCard())){
                    List<RosterBase> rosterBaseList = this.list(new QueryWrapper<RosterBase>().eq("id_card", rosterBaseDTO.getIdCard()));
                    if(CollectionUtil.isNotEmpty(rosterBaseList)){
                        return ResponseEntity.badRequest().body("基础信息-身份证号 已存在，保存失败");
                    }
                }
                // 手机号唯一验证
                if(StringUtils.isNotBlank(rosterBaseDTO.getMobile())){
                    List<RosterBase> rosterBaseList = this.list(new QueryWrapper<RosterBase>().eq("mobile", rosterBaseDTO.getMobile()));
                    if(CollectionUtil.isNotEmpty(rosterBaseList)){
                        return ResponseEntity.badRequest().body("基础信息-手机号码 已存在，保存失败");
                    }
                }
            }
            if ("1".equals(rosterBaseDTO.getPersonType())) { // 当前人员是员工时
                //获取当前登录人信息
                UserDTO userDTO = UserUtils.getCurrentUserDTO();
                //工号生成
                String serialNum = serialnumTplService.genSerialNum(userDTO.getCompanyDTO().getId(), RosterBaseDTO.BIZ_CODE);
                if (StringUtils.isNotBlank(rosterBaseDTO.getId())) {
                    RosterBase byId = this.getById(rosterBaseDTO.getId());
                    if (StringUtils.isBlank(byId.getJobNo())) {
                        rosterBaseDTO.setJobNo(serialNum);  // 游客转员工，保存时，生成工号
                    }
                } else {
                    rosterBaseDTO.setJobNo(serialNum);  // 员工新增生成工号
                }
            }
        }


        RosterBase rosterBase = RosterBaseWrapper.INSTANCE.toEntity(rosterBaseDTO);



        // 新增或修改花名册数据
        this.saveOrUpdate(rosterBase);
        rosterBase.setNewPassword(rosterBaseDTO.getNewPassword());
        rosterBase.setLoginFlag(rosterBaseDTO.getLoginFlag());
        rosterBase.setOtherServiceFlag(rosterBaseDTO.getOtherServiceFlag());

        // 新增或修改用户数据(花名册的人员类型为员工时)
        if ("1".equals(rosterBase.getPersonType())) {
            String userId = this.saveOrUpdateUser(rosterBase, CommonUtils.getFirstCompanyId());
            rosterBase.setUserId(userId);
        }

        // 推送其他系统同步保存用户信息
        syncPushSubsystem(rosterBase, oldOfficeId);

        if(ObjectUtil.isNotEmpty(rosterBaseDTO)){
            //保存合同信息
            if(ObjectUtil.isNotEmpty(rosterBaseDTO.getRosterContractDTO())){
                RosterContract rosterContract = RosterContractWrapper.INSTANCE.toEntity(rosterBaseDTO.getRosterContractDTO());
                rosterContract.setBaseId(rosterBase.getId());
                rosterContractService.remove(new QueryWrapper<RosterContract>().eq("base_id",rosterBase.getId()));
                rosterContractService.saveOrUpdate(rosterContract);
            } else {
                rosterContractService.remove(new QueryWrapper<RosterContract>().eq("base_id",rosterBase.getId()));
            }
            //保存银行卡信息
            if(CollectionUtil.isNotEmpty(rosterBaseDTO.getRosterBankCardsDTOList())){
                List<String> collect = rosterBaseDTO.getRosterBankCardsDTOList().stream().map(RosterBankCardsDTO::getId).collect(Collectors.toList());
                rosterBankCardsService.remove(new QueryWrapper<RosterBankCards>().eq("base_id",rosterBase.getId()).notIn(CollectionUtil.isNotEmpty(collect),"id",collect));
                rosterBaseDTO.getRosterBankCardsDTOList().stream().forEach(item->{
                    if(ObjectUtil.isNotEmpty(item)&& BeanUtils.objectCheckIsNull(item)){
                        RosterBankCards rosterBankCards = RosterBankCardsWrapper.INSTANCE.toEntity(item);
                        rosterBankCards.setBaseId(rosterBase.getId());
                        rosterBankCardsService.saveOrUpdate(rosterBankCards);
                    }
                });
            }else{
                rosterBankCardsService.remove(new QueryWrapper<RosterBankCards>().eq("base_id",rosterBase.getId()));
            }
            //保存教育经历
            if(CollectionUtil.isNotEmpty(rosterBaseDTO.getRosterEducateDTOList())){
                List<String> collect = rosterBaseDTO.getRosterEducateDTOList().stream().map(RosterEducateDTO::getId).collect(Collectors.toList());
                rosterEducateService.remove(new QueryWrapper<RosterEducate>().eq("base_id",rosterBase.getId()).notIn(CollectionUtil.isNotEmpty(collect),"id",collect));
                rosterBaseDTO.getRosterEducateDTOList().stream().forEach(item->{
                    if(ObjectUtil.isNotEmpty(item)&& BeanUtils.objectCheckIsNull(item)){
                        RosterEducate rosterEducate = RosterEducateWrapper.INSTANCE.toEntity(item);
                        rosterEducate.setBaseId(rosterBase.getId());
                        rosterEducateService.saveOrUpdate(rosterEducate);
                    }
                });
            }else{
                rosterEducateService.remove(new QueryWrapper<RosterEducate>().eq("base_id",rosterBase.getId()));
            }
            //保存工作经历
            if(CollectionUtil.isNotEmpty(rosterBaseDTO.getRosterWorkDTOList())){
                List<String> collect = rosterBaseDTO.getRosterWorkDTOList().stream().map(RosterWorkDTO::getId).collect(Collectors.toList());
                rosterWorkService.remove(new QueryWrapper<RosterWork>().eq("base_id",rosterBase.getId()).notIn(CollectionUtil.isNotEmpty(collect),"id",collect));
                rosterBaseDTO.getRosterWorkDTOList().stream().forEach(item->{
                    if(ObjectUtil.isNotEmpty(item)&& BeanUtils.objectCheckIsNull(item)){
                        RosterWork rosterWork = RosterWorkWrapper.INSTANCE.toEntity(item);
                        rosterWork.setBaseId(rosterBase.getId());
                        rosterWorkService.saveOrUpdate(rosterWork);
                    }
                });
            }else{
                rosterWorkService.remove(new QueryWrapper<RosterWork>().eq("base_id",rosterBase.getId()));
            }
            //保存家庭成员
            if(CollectionUtil.isNotEmpty(rosterBaseDTO.getRosterFamilyDTOList())){
                List<String> collect = rosterBaseDTO.getRosterFamilyDTOList().stream().map(RosterFamilyDTO::getId).collect(Collectors.toList());
                rosterFamilyService.remove(new QueryWrapper<RosterFamily>().eq("base_id",rosterBase.getId()).notIn(CollectionUtil.isNotEmpty(collect),"id",collect));
                rosterBaseDTO.getRosterFamilyDTOList().stream().forEach(item->{
                    if(ObjectUtil.isNotEmpty(item)&& BeanUtils.objectCheckIsNull(item)){
                        RosterFamily rosterFamily = RosterFamilyWrapper.INSTANCE.toEntity(item);
                        rosterFamily.setBaseId(rosterBase.getId());
                        rosterFamilyService.saveOrUpdate(rosterFamily);
                    }
                });
            }else{
                rosterFamilyService.remove(new QueryWrapper<RosterFamily>().eq("base_id",rosterBase.getId()));
            }
            //保存员工证书
            if(CollectionUtil.isNotEmpty(rosterBaseDTO.getRosterCertificateDTOList())){
                List<String> collect = rosterBaseDTO.getRosterCertificateDTOList().stream().map(RosterCertificateDTO::getId).collect(Collectors.toList());
                rosterCertificateService.remove(new QueryWrapper<RosterCertificate>().eq("base_id",rosterBase.getId()).notIn(CollectionUtil.isNotEmpty(collect),"id",collect));
                rosterBaseDTO.getRosterCertificateDTOList().stream().forEach(item->{
                    if(ObjectUtil.isNotEmpty(item)&& BeanUtils.objectCheckIsNull(item)){
                        RosterCertificate rosterCertificate = RosterCertificateWrapper.INSTANCE.toEntity(item);
                        rosterCertificate.setBaseId(rosterBase.getId());
                        rosterCertificateService.saveOrUpdate(rosterCertificate);
                    }
                });
            }else{
                rosterCertificateService.remove(new QueryWrapper<RosterCertificate>().eq("base_id",rosterBase.getId()));
            }
            //保存附件
            if(CollectionUtil.isNotEmpty(rosterBaseDTO.getRosterFilesDTOList())){
                List<String> collect = rosterBaseDTO.getRosterFilesDTOList().stream().distinct().filter(item->{
                    if(StringUtils.isNotBlank(item.getId())){
                        return true;
                    }
                    return false;
                }).map(RosterFilesDTO::getId).collect(Collectors.toList());
                rosterFilesService.remove(new QueryWrapper<RosterFiles>().eq("base_id",rosterBase.getId()).notIn(CollectionUtil.isNotEmpty(collect),"id",collect));
                rosterBaseDTO.getRosterFilesDTOList().stream().forEach(item->{
                    if(ObjectUtil.isNotEmpty(item)&& BeanUtils.objectCheckIsNull(item)){
                        RosterFiles rosterFiles = RosterFilesWrapper.INSTANCE.toEntity(item);
                        rosterFiles.setBaseId(rosterBase.getId());
                        if(StringUtils.isNotBlank(item.getCreateBy().getId())){
                            rosterFiles.setCreateBy(item.getCreateBy().getId());
                        }
                        rosterFilesService.saveOrUpdate(rosterFiles);
                    }
                });
            }else{
                rosterFilesService.remove(new QueryWrapper<RosterFiles>().eq("base_id",rosterBase.getId()));
            }
            //保存直属领导
//            if(ObjectUtil.isNotEmpty(rosterBaseDTO.getRosterLeadershipDTO())){
//                RosterLeadership rosterLeadership = RosterLeadershipWrapper.INSTANCE.toEntity(rosterBaseDTO.getRosterLeadershipDTO());
//                rosterLeadership.setBaseId(rosterBase.getId());
//                rosterLeadershipService.remove(new QueryWrapper<RosterLeadership>().eq("base_id",rosterBase.getId()));
//                rosterLeadershipService.saveOrUpdate(rosterLeadership);
//            }else{
//                rosterLeadershipService.remove(new QueryWrapper<RosterLeadership>().eq("base_id",rosterBase.getId()));
//            }
            User user = userService.getOne(new QueryWrapper<User>().lambda().eq(User::getRosterId, rosterBase.getId()));
            if (ObjectUtil.isNotEmpty(user)) {
                // 保存花名册关联用户的资质信息
//                certService.remove(new QueryWrapper<Cert>().lambda().eq(Cert::getUserId,user.getId()));
//                if(ObjectUtil.isNotEmpty(rosterBaseDTO.getCertDTOList())){
//                    List<Cert> certList = new ArrayList<>();
//                    rosterBaseDTO.getCertDTOList().stream().forEach(i -> {
//                        Cert cert = CertWrapper.INSTANCE.toEntity(i);
//                        cert.setUserId(user.getId());
//                        cert.setId("");
//                        certList.add(cert);
//                    });
//                    if (CollectionUtil.isNotEmpty(certList)) {
//                        certService.saveBatch(certList);
//                    }
//                }
                // 保存花名册关联用户的角色
                userMapper.deleteUserRole(user.getId());  //  删除用户的角色
                if (CollectionUtil.isNotEmpty(rosterBaseDTO.getRoleIdList())) {
                    rosterBaseDTO.getRoleIdList().stream().forEach(i -> {
                        userMapper.insertUserRole(user.getId(), i);  // 插入用户的角色
                    });
                }
                // 保存花名册关联用户的岗位
                userMapper.deleteUserPost(user.getId());  //  删除用户的岗位
                if (CollectionUtil.isNotEmpty(rosterBaseDTO.getPostIdList())) {
                    rosterBaseDTO.getPostIdList().stream().forEach(i -> {
                        userMapper.insertUserPost(user.getId(), i);  // 插入用户的岗位
                    });
                }
                // 保存花名册关联用户的管理部门
                UserDTO u = new UserDTO();
                u.setId(user.getId());
                userMapper.deleteUserManageOffice(u);  //  删除用户的管理部门 （用户、管理部门关联表）
                String oIds = "";
                if (StringUtils.isNotBlank(rosterBaseDTO.getManageOfficeIds())) {
                    List<String> officeIds= Arrays.asList(rosterBaseDTO.getManageOfficeIds().split(","));
                    officeIds.stream().forEach(officeId -> {
                        userMapper.insertUserManageOffice(officeId,user.getId());  // 插入用户的管理部门 （用户、管理部门关联表）
                    });
                    oIds = rosterBaseDTO.getManageOfficeIds();
                }
                userMapper.updateUserManageOffice(oIds, user.getId()); // 修改用户的管理部门 （用户表）
            }
        }
        return ResponseEntity.ok("花名册保存成功");
    }

    /**
     * 推送其他系统同步保存用户信息
     * @param rosterBase
     */
    public void syncPushSubsystem(RosterBase rosterBase, String oldOfficeId) {
        // 接口所需参数
        Map<String,Object> bodyMap = JSONObject.parseObject(JSON.toJSONString(rosterBase), Map.class);
        // 判断是否有修改用户的所属部门(oldOfficeId为空，则直接进行保存推送)。
        // 没有修改的话，则直接进行保存推送。有修改的话，要判断是否所属同一个子系统。
        // 有修改的话。所属是同一个子系统 则直接进行保存推送，不是 则要清空旧所属系统的用户，再保存到新所属系统中
        String pushType = "";
        if (StringUtils.isNotBlank(oldOfficeId) && !oldOfficeId.equals(rosterBase.getOfficeId())) {
            String officeCode1 = getServiceByOffice(oldOfficeId);
            String officeCode2 = getServiceByOffice(rosterBase.getOfficeId());
            if (StringUtils.isNotBlank(officeCode1) && StringUtils.isNotBlank(officeCode2) && !officeCode1.equals(officeCode2) && !ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCode2)) {
                // 有修改用户的所属部门，并且所属不是同一个子系统，修改后的所属部门不是公共部门时，则要清除旧所属系统的该用户，再保存到新所属系统中
                pushType = "clear";
            }
        }
        // 根据所属部门来判断，要向哪个子系统推送保存操作
        // 根据部门id找出该部门的二级部门，获取这个二级部门的别名code
        String officeCode = getServiceByOffice(rosterBase.getOfficeId());
        // 清除旧所属系统的该用户
        if (StringUtils.isNotBlank(pushType)&&pushType.equals("clear")) {
            String officeCodeOld = getServiceByOffice(oldOfficeId);  // 根据这个部门id获取二级部门的别名code来获取所属哪个子系统
            if (StringUtils.isNotBlank(officeCodeOld)) {
                // 评估
                if (ServiceAliasEnum.CPA_EVALUATION.getValue().equals(officeCodeOld) || (ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCodeOld) && !ServiceAliasEnum.CPA_EVALUATION.getValue().equals(officeCode))) {
                    // 首先将数据记录为清除失败，待获取到其他系统执行成功的回值后再将此记录删掉
                    String dataId = sysUserFailedLogService.deleteFailedLog(rosterBase.getUserId(), ServiceAliasEnum.CPA_EVALUATION.getValue());
                    // 调用其他系统的用户删除操作
                    Object res = restTemplateService.postCPA_EVALUATION("/roster/base/deleteUserRoster",null,null,bodyMap);
                    if (ObjectUtil.isNotEmpty(res)) {
                        Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res), Map.class);
                        if ((Boolean) resMap.get("success")) {
                            // 保存成功，将刚才记录的数据删除
                            sysUserFailedLogService.removeById(dataId);
                        }
                    }
                }
                // 财务
                if (ServiceAliasEnum.CPA_FINANCE.getValue().equals(officeCodeOld) || (ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCodeOld) && !ServiceAliasEnum.CPA_FINANCE.getValue().equals(officeCode))) {
                    // 首先将数据记录为清除失败，待获取到其他系统执行成功的回值后再将此记录删掉
                    String dataId = sysUserFailedLogService.deleteFailedLog(rosterBase.getUserId(), ServiceAliasEnum.CPA_FINANCE.getValue());
                    // 调用其他系统的用户删除操作
                    Object res = restTemplateService.postCPA_FINANCE("/roster/base/deleteUserRoster",null,null,bodyMap);
                    if (ObjectUtil.isNotEmpty(res)) {
                        Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res), Map.class);
                        if ((Boolean) resMap.get("success")) {
                            // 保存成功，将刚才记录的数据删除
                            sysUserFailedLogService.removeById(dataId);
                        }
                    }
                }
                // 中审
                if (ServiceAliasEnum.CPA_AUDIT.getValue().equals(officeCodeOld) || (ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCodeOld)) && !ServiceAliasEnum.CPA_AUDIT.getValue().equals(officeCode)) {
                    // 首先将数据记录为清除失败，待获取到其他系统执行成功的回值后再将此记录删掉
                    String dataId = sysUserFailedLogService.deleteFailedLog(rosterBase.getUserId(), ServiceAliasEnum.CPA_AUDIT.getValue());
                    // 调用其他系统的用户删除操作
                    Object res = restTemplateService.postCPA_AUDIT("/roster/base/deleteUserRoster",null,null,bodyMap);
                    if (ObjectUtil.isNotEmpty(res)) {
                        Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res), Map.class);
                        if ((Boolean) resMap.get("success")) {
                            // 保存成功，将刚才记录的数据删除
                            sysUserFailedLogService.removeById(dataId);
                        }
                    }
                }
            }
        }

        // 向子系统推送保存操作
        if (StringUtils.isNotBlank(officeCode)) {
            // 评估
            if (ServiceAliasEnum.CPA_EVALUATION.getValue().equals(officeCode) || ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCode)) {
                // 首先将数据记录为保存失败，待获取到其他系统执行成功的回值后再将此记录删掉
                String dataId = sysUserFailedLogService.saveFailedLog(rosterBase.getUserId(), ServiceAliasEnum.CPA_EVALUATION.getValue());
                // 调用其他系统的用户保存操作
                Object res = restTemplateService.postCPA_EVALUATION("/roster/base/saveOrUpdateUserRoster",null,null,bodyMap);
                if (ObjectUtil.isNotEmpty(res)) {
                    Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res), Map.class);
                    if ((Boolean) resMap.get("success")) {
                        // 保存成功，将刚才记录的数据删除
                        sysUserFailedLogService.removeById(dataId);
                    }
                }
            }
            // 财务
            if (ServiceAliasEnum.CPA_FINANCE.getValue().equals(officeCode) || ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCode)) {
                // 首先将数据记录为保存失败，待获取到其他系统执行成功的回值后再将此记录删掉
                String dataId = sysUserFailedLogService.saveFailedLog(rosterBase.getUserId(), ServiceAliasEnum.CPA_FINANCE.getValue());
                // 调用其他系统的用户保存操作
                Object res = restTemplateService.postCPA_FINANCE("/roster/base/saveOrUpdateUserRoster",null,null,bodyMap);
                if (ObjectUtil.isNotEmpty(res)) {
                    Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res), Map.class);
                    if ((Boolean) resMap.get("success")) {
                        // 保存成功，将刚才记录的数据删除
                        sysUserFailedLogService.removeById(dataId);
                    }
                }
            }
            // 中审
            if (ServiceAliasEnum.CPA_AUDIT.getValue().equals(officeCode) || ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCode)) {
                // 首先将数据记录为保存失败，待获取到其他系统执行成功的回值后再将此记录删掉
                String dataId = sysUserFailedLogService.saveFailedLog(rosterBase.getUserId(), ServiceAliasEnum.CPA_AUDIT.getValue());
                // 调用其他系统的用户保存操作
                Object res = restTemplateService.postCPA_AUDIT("/roster/base/saveOrUpdateUserRoster",null,null,bodyMap);
                if (ObjectUtil.isNotEmpty(res)) {
                    Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res), Map.class);
                    if ((Boolean) resMap.get("success")) {
                        // 保存成功，将刚才记录的数据删除
                        sysUserFailedLogService.removeById(dataId);
                    }
                }
            }
        }
    }

    /**
     * 根据部门id找出该部门的二级部门(包含此部门就是二级部门的可能)，然后获取这个二级部门的别名code
     * @param officeId
     * @return
     */
    public String getServiceByOffice(String officeId) {
        String officeAlias = "";
        Office office = officeService.getById(officeId);
        if (ObjectUtil.isNotEmpty(office)) {
            if (StringUtils.isNotBlank(office.getIsPublic()) && "1".equals(office.getIsPublic())) {
                officeAlias = ServiceAliasEnum.CPA_PUBLIC.getValue(); // 共有部门
            }else if (StringUtils.isNotBlank(office.getGrade()) && "2".equals(office.getGrade())) {
                officeAlias =  StringUtils.isNotBlank(office.getCode()) ? office.getCode() : "";
            } else {
                if (StringUtils.isNotBlank(office.getParentIds())) {
                    String[] split = office.getParentIds().split(",");
                    List<String> ids = Arrays.asList(split);
                    for (String oId : ids) {
                        Office o = officeService.getById(oId);
                        if (ObjectUtil.isNotEmpty(o) && StringUtils.isNotBlank(o.getGrade()) && "2".equals(o.getGrade())) {
                            officeAlias =  StringUtils.isNotBlank(o.getCode()) ? o.getCode() : "";
                        }
                    }
                }
            }
        }
        return officeAlias;
    }

    // 导入花名册excel
    public ResponseEntity importFile(MultipartFile file) {
        try {
            Integer successNum = 0;
            List<RosterBase> rosterList = new ArrayList<>();
            // 获取上传文件中的数据
            List <RosterBaseDTO> list = EasyPoiUtil.importExcel ( file, 1, 1, RosterBaseDTO.class );
            //去除excel中的空行
            list = getExcelList(list);
            for (RosterBaseDTO rosterBaseDTO : list) {
                // 姓名为必填项
                if (StringUtils.isBlank(rosterBaseDTO.getName())) {
                    return ResponseEntity.badRequest().body ("导入花名册失败！文件中有姓名未填写");
                }
                // 姓名唯一验证
//                List<RosterBase> rs = this.list(new QueryWrapper<RosterBase>().lambda().eq(RosterBase::getName, rosterBaseDTO.getName()));
//                if (CollectionUtil.isNotEmpty(rs)) {
//                    return ResponseEntity.badRequest().body ("导入花名册失败！人员“"+ rosterBaseDTO.getName() +"”已存在，请重新填写");
//                }

                // 人员类型必填
                if (StringUtils.isBlank(rosterBaseDTO.getPersonType())) {
                    return ResponseEntity.badRequest().body ("导入花名册失败！文件中有人员类型未填写");
                }
                // 验证人员类型填写是否符合要求
                String personType = DictUtils.getDictValue(rosterBaseDTO.getPersonType(), "roster_person_type","null");
                if ("null".equals(personType)) {
                    return ResponseEntity.badRequest().body ("导入花名册失败！文件中有人员类型填写错误，请重新填写");
                }
                // 人员类型转换
                rosterBaseDTO.setPersonType(personType);

                // 人员类型为员工
                if ("1".equals(personType)) {
                    if (StringUtils.isBlank(rosterBaseDTO.getOfficeName())) {
                        return ResponseEntity.badRequest().body ("导入花名册失败！人员类型为员工的时候，部门为必填项");
                    }
                    if (StringUtils.isBlank(rosterBaseDTO.getCertificatesType())) {
                        return ResponseEntity.badRequest().body ("导入花名册失败！人员类型为员工的时候，证件类型为必填项");
                    }
                    if (StringUtils.isBlank(rosterBaseDTO.getIdCard())) {
                        return ResponseEntity.badRequest().body ("导入花名册失败！人员类型为员工的时候，证件号为必填项");
                    }
                    if (StringUtils.isBlank(rosterBaseDTO.getWorkerType())) {
                        return ResponseEntity.badRequest().body ("导入花名册失败！人员类型为员工的时候，员工类型为必填项");
                    }
                } else {
                    // 人员类型为 游客 的时候，不填写部门和员工类型
                    rosterBaseDTO.setOfficeName("");
                    rosterBaseDTO.setWorkerType("");
                }

                if (StringUtils.isNotBlank(rosterBaseDTO.getOfficeName())) {
                    // 查询这个部门是否存在于当前系统中，不存在则不进行导入数据操作，抛出提示
                    List<Office> officeList = officeService.list(new QueryWrapper<Office>().lambda().eq(Office::getType, OfficeTypeEnum.OFFICE.getValue()).eq(Office::getName, rosterBaseDTO.getOfficeName()));
                    if (CollectionUtil.isEmpty(officeList)) {
                        return ResponseEntity.badRequest().body("导入花名册失败！文件中的部门”" + rosterBaseDTO.getOfficeName() + "“不存在，请重新填写");
                    }
                    // 部门填写符合要求，将部门名字转换为部门id
                    rosterBaseDTO.setOfficeId(officeList.get(0).getId());
                }

                if (StringUtils.isNotBlank(rosterBaseDTO.getCertificatesType())) {
                    // 验证证件类型填写是否符合要求
                    String certificatesType = DictUtils.getDictValue(rosterBaseDTO.getCertificatesType(), "certificates_type","null");
                    if ("null".equals(certificatesType)) {
                        return ResponseEntity.badRequest().body ("导入花名册失败！文件中有证件类型填写错误，请重新填写");
                    }
                    // 证件类型转换
                    rosterBaseDTO.setCertificatesType(certificatesType);
                }

                if (StringUtils.isNotBlank(rosterBaseDTO.getIdCard())) {
                    // 证件号唯一验证
                    List<RosterBase> rosterBaseList = this.list(new QueryWrapper<RosterBase>().eq("id_card", rosterBaseDTO.getIdCard()));
                    if(CollectionUtil.isNotEmpty(rosterBaseList)){
                        return ResponseEntity.badRequest().body("导入花名册失败！文件中证件号”" + rosterBaseDTO.getIdCard() + "“已存在，请重新填写");
                    }
                }

                // 电话号码必填
                if (StringUtils.isBlank(rosterBaseDTO.getMobile())) {
                    return ResponseEntity.badRequest().body ("导入花名册失败！文件中有电话号码未填写");
                }
                // 电话号码唯一验证
                List<RosterBase> mobileList = this.list(new QueryWrapper<RosterBase>().lambda().eq(RosterBase::getMobile, rosterBaseDTO.getMobile()));
                if (CollectionUtil.isNotEmpty(mobileList)) {
                    return ResponseEntity.badRequest().body ("导入花名册失败！文件中有电话号码已存在，请重新填写");
                }

                if (StringUtils.isNotBlank(rosterBaseDTO.getWorkerType())) {
                    // 验证员工类型填写是否符合要求
                    String workerType = DictUtils.getDictValue(rosterBaseDTO.getWorkerType(), "worker_type","null");
                    if ("null".equals(workerType)) {
                        return ResponseEntity.badRequest().body ("导入花名册失败！文件中有员工类型填写错误，请重新填写");
                    }
                    // 员工类型转换
                    rosterBaseDTO.setWorkerType(workerType);
                }

                // 生成最终数据
                RosterBase rosterBase = RosterBaseWrapper.INSTANCE.toEntity(rosterBaseDTO);
                rosterList.add(rosterBase);
                successNum++;
            }
            if (CollectionUtil.isNotEmpty(rosterList)) {
                String companyId = CommonUtils.getFirstCompanyId();
                // 批量新增花名册数据
                this.saveBatch(rosterList);
                rosterList.stream().forEach(r -> {
                    // 人员类型为员工时,同步新增用户信息
                    if ("1".equals(r.getPersonType())) {
                        this.saveOrUpdateUser(r, companyId);
                    }
                });
            }
            return ResponseEntity.ok ( "已成功导入 " + successNum + " 条数据" );
        } catch (Exception e) {
            return ResponseEntity.badRequest().body ( "导入花名册失败！失败信息：" + e.getMessage ( ) );
        }
    }
    /**
     * 去除excel中的空行
     * @param list
     * @return
     */
    public ArrayList<RosterBaseDTO> getExcelList(List<RosterBaseDTO> list){

        ArrayList<RosterBaseDTO> rosterBaseDTOS = new ArrayList<>();

        list.stream().forEach(item->{
            if(objectCheckIsNull(item)){
                rosterBaseDTOS.add(item);
            }
        });

        return rosterBaseDTOS;
    }


    /**
     * 新增或修改花名册的时候，同步创建或修改用户数据
     */
    public String saveOrUpdateUser (RosterBase rosterBase,String companyId) {
        User user = new User();
        user.setRosterId(rosterBase.getId());  // 花名册id
        user.setOfficeId(rosterBase.getOfficeId());  //  部门
        user.setMobile(rosterBase.getMobile());  //  电话号码
        user.setName(rosterBase.getName());  //  姓名
        user.setLoginName(rosterBase.getName());  //  登录名
        user.setNo(rosterBase.getJobNo());  //  工号
        if (StringUtils.isNotBlank(rosterBase.getEmail())) {
            user.setEmail(rosterBase.getEmail()); // 邮箱
        }
        if (StringUtils.isNotBlank(rosterBase.getCompanyId())) { // 公司
            user.setCompanyId(rosterBase.getCompanyId());
        } else {
            user.setCompanyId(companyId);  // 默认给第一个公司
        }
        //  根据花名册id查询用户信息
        User u = userService.getOne(new QueryWrapper<User>().lambda().eq(User::getRosterId, rosterBase.getId()));
        if (ObjectUtil.isNotEmpty(u)) {
            user.setId(u.getId());  //  用户id
            user.setUpPassword("1");  //  密码修改状态改为 ”已修改“
            // 密码
            if (StringUtils.isNotBlank(rosterBase.getNewPassword())) {
                user.setPassword(SecurityUtils.encryptPassword(rosterBase.getNewPassword()));
            }
            //  是否允许登录
            if (StringUtils.isNotBlank(rosterBase.getLoginFlag())) {
                user.setLoginFlag(rosterBase.getLoginFlag());
            }
            // 访问其他系统权限
            if (StringUtils.isNotBlank(rosterBase.getOtherServiceFlag())) {
                user.setOtherServiceFlag(rosterBase.getOtherServiceFlag());
            }
        } else {
            //  是否允许登录
            if (StringUtils.isNotBlank(rosterBase.getLoginFlag())) {
                user.setLoginFlag(rosterBase.getLoginFlag());
            } else {
                user.setLoginFlag("1");  //  是否允许登录     默认直接给 “1” 允许登录
            }
            // 访问其他系统权限
            if (StringUtils.isNotBlank(rosterBase.getOtherServiceFlag())) {
                user.setOtherServiceFlag(rosterBase.getOtherServiceFlag());
            } else {
                user.setOtherServiceFlag("0");  //  访问其他系统权限     默认直接给 “0” 不允许访问其他系统
            }
            //  密码
            if (StringUtils.isNotBlank(rosterBase.getNewPassword())) {
                user.setPassword(SecurityUtils.encryptPassword(rosterBase.getNewPassword()));
            } else {
                user.setPassword(SecurityUtils.encryptPassword("123456"));  //  密码     默认密码 “123456”
            }
        }
        userService.saveOrUpdate(user);
        return user.getId();
    }

    public ResponseEntity removeByRosterIds(String[] ids) {
        List<String> rosterIds = Arrays.asList(ids);
        AtomicBoolean runSuccess = new AtomicBoolean(true);
        // 先删除子系统花名册以及用户数据
        rosterIds.stream().forEach(rosterId -> {
            Map<String,Object> paramMap = new HashMap<>();
            paramMap.put("rosterId",rosterId);
            User u = userService.getOne(new QueryWrapper<User>().lambda().eq(User::getRosterId, rosterId));
            paramMap.put("userId",u.getId());
            String officeCode = getServiceByOffice(u.getOfficeId());  // 根据这个部门id获取二级部门的别名code来获取所属哪个子系统
            if (StringUtils.isNotBlank(officeCode)) {
                // 评估
                if (ServiceAliasEnum.CPA_EVALUATION.getValue().equals(officeCode) || ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCode)) {
                    // 首先将数据记录为清除失败，待获取到其他系统执行成功的回值后再将此记录删掉
                    boolean succ = false;
                    String dataId_evaluation = sysUserFailedLogService.removeFailedLog(u.getId(), ServiceAliasEnum.CPA_EVALUATION.getValue());
                    Object res_evaluation = restTemplateService.getCPA_EVALUATION("/roster/base/removeByRosterId",null,paramMap);
                    if (ObjectUtil.isNotEmpty(res_evaluation)) {
                        Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res_evaluation), Map.class);
                        if ((Boolean) resMap.get("success")) {
                            // 删除成功，将刚才记录的数据删除
                            sysUserFailedLogService.removeById(dataId_evaluation);
                            succ = true;
                        }
                    }
                    if (!succ) { // succ的值为false，那么说明数据删除失败
                        runSuccess.set(false);
                    }
                }
                // 财务
                if (ServiceAliasEnum.CPA_FINANCE.getValue().equals(officeCode) || ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCode)) {
                    // 首先将数据记录为清除失败，待获取到其他系统执行成功的回值后再将此记录删掉
                    boolean succ = false;
                    String dataId_finance = sysUserFailedLogService.removeFailedLog(u.getId(), ServiceAliasEnum.CPA_FINANCE.getValue());
                    Object res_finance = restTemplateService.getCPA_FINANCE("/roster/base/removeByRosterId",null,paramMap);
                    if (ObjectUtil.isNotEmpty(res_finance)) {
                        Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res_finance), Map.class);
                        if ((Boolean) resMap.get("success")) {
                            // 删除成功，将刚才记录的数据删除
                            sysUserFailedLogService.removeById(dataId_finance);
                            succ = true;
                        }
                    }
                    if (!succ) { // succ的值为false，那么说明数据删除失败
                        runSuccess.set(false);
                    }
                }
                // 中审
                if (ServiceAliasEnum.CPA_AUDIT.getValue().equals(officeCode) || ServiceAliasEnum.CPA_PUBLIC.getValue().equals(officeCode)) {
                    // 首先将数据记录为清除失败，待获取到其他系统执行成功的回值后再将此记录删掉
                    boolean succ = false;
                    String dataId_audit = sysUserFailedLogService.removeFailedLog(u.getId(), ServiceAliasEnum.CPA_AUDIT.getValue());
                    Object res_audit = restTemplateService.getCPA_AUDIT("/roster/base/removeByRosterId",null,paramMap);
                    if (ObjectUtil.isNotEmpty(res_audit)) {
                        Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res_audit), Map.class);
                        if ((Boolean) resMap.get("success")) {
                            // 删除成功，将刚才记录的数据删除
                            sysUserFailedLogService.removeById(dataId_audit);
                            succ = true;
                        }
                    }
                    if (!succ) { // succ的值为false，那么说明数据删除失败
                        runSuccess.set(false);
                    }
                }
            }
        });
        Map<String,Object> resMap = new HashMap<>();
        if (runSuccess.get()) { // runSuccess的值为true，则说明子系统的操作都执行成功
            // 删除花名册数据
            this.removeByIds(rosterIds);
            // 删除花名册关联用户数据。删除用户信息以及用户关联的角色等信息
            rosterIds.stream().forEach(rosterId -> {
                // 通过花名册id查询用户
                User u = userService.getOne(new QueryWrapper<User>().lambda().eq(User::getRosterId, rosterId));
                if (ObjectUtil.isNotEmpty(u)) {
                    // 删除用户
                    userService.removeById(u.getId());
                    UserDTO userDTO = new UserDTO();
                    userDTO.setId(u.getId());
                    // 删除用户关联的管理部门信息
                    userMapper.deleteUserManageOffice(userDTO);  //  根据用户id删除用户的管理部门信息
                    // 删除用户关联的岗位信息
                    userMapper.deleteUserPost(u.getId());
                    // 删除用户关联的角色信息
                    userMapper.deleteUserRole(u.getId());
                }
            });
            resMap.put("success", true);
            resMap.put("message", "删除花名册成功");
        } else {
            resMap.put("success", false);
            resMap.put("message", "删除花名册失败，子系统删除数据未成功");
        }
        return ResponseEntity.ok(resMap);
    }

    public RosterBaseDTO deleteRosterWorker (RosterBaseDTO rosterBaseDTO) {
//        删除原数据关联表的数据
        rosterBaseDTO.setRosterContractDTO(null);  // 删除合同信息
        rosterBaseDTO.setRosterBankCardsDTOList(null);  // 删除银行卡信息
        rosterBaseDTO.setRosterEducateDTOList(null);  // 删除教育经历
        rosterBaseDTO.setRosterWorkDTOList(null);  // 删除工作经历
        rosterBaseDTO.setRosterFamilyDTOList(null);  // 删除家庭成员
        rosterBaseDTO.setRosterCertificateDTOList(null);  // 删除员工证书
        rosterBaseDTO.setRosterLeadershipDTO(null);  // 删除直属领导
        rosterBaseDTO.setCertDTOList(null);  //  关联用户的资质
        rosterBaseDTO.setRoleIdList(null);  // 关联用户的角色
        rosterBaseDTO.setPostIdList(null);  //  关联用户的岗位
        rosterBaseDTO.setManageOfficeIds(null);  //  关联用户的管理部门
        // 与游客无关字段置空
        rosterBaseDTO.setOfficeId("");
        rosterBaseDTO.setJobPositionId("");
        rosterBaseDTO.setPositionId("");
        rosterBaseDTO.setRankId("");
        rosterBaseDTO.setJobNo("");
        rosterBaseDTO.setContractType("");
        rosterBaseDTO.setWorkerType("");
        rosterBaseDTO.setOnboardingDate(null);
        rosterBaseDTO.setSeniority("");
        rosterBaseDTO.setContractCompany("");
        rosterBaseDTO.setBirthDate(null);
        rosterBaseDTO.setWorkPlace("");
        rosterBaseDTO.setContactAddress("");
        rosterBaseDTO.setEmploymentSituation("");
        rosterBaseDTO.setWorkingHour("");
        rosterBaseDTO.setAccountType("");
        rosterBaseDTO.setAccountAddr("");
        rosterBaseDTO.setOrigin("");
        rosterBaseDTO.setLiveAddr("");
        rosterBaseDTO.setHighestEducation("");
        rosterBaseDTO.setPoliticalFace("");
        rosterBaseDTO.setMaritalStatus("");
        rosterBaseDTO.setEmergencyContactName("");
        rosterBaseDTO.setEmergencyContactTel("");
        rosterBaseDTO.setSpouseName("");
        rosterBaseDTO.setChildName("");
        rosterBaseDTO.setHeadshot("");
        // 表删除无关字段
        UpdateWrapper<RosterBase> wrapper = new UpdateWrapper<>();
        wrapper.lambda()
                .set(RosterBase::getOfficeId,null)
                .set(RosterBase::getJobPositionId,null)
                .set(RosterBase::getPositionId,null)
                .set(RosterBase::getRankId,null)
                .set(RosterBase::getJobNo,null)
                .set(RosterBase::getContractType,null)
                .set(RosterBase::getWorkerType,null)
                .set(RosterBase::getOnboardingDate,null)
                .set(RosterBase::getSeniority,null)
                .set(RosterBase::getContractCompany,null)
                .set(RosterBase::getBirthDate,null)
                .set(RosterBase::getWorkPlace,null)
                .set(RosterBase::getContactAddress,null)
                .set(RosterBase::getEmploymentSituation,null)
                .set(RosterBase::getWorkingHour,null)
                .set(RosterBase::getAccountType,null)
                .set(RosterBase::getAccountAddr,null)
                .set(RosterBase::getOrigin,null)
                .set(RosterBase::getLiveAddr,null)
                .set(RosterBase::getHighestEducation,null)
                .set(RosterBase::getPoliticalFace,null)
                .set(RosterBase::getMaritalStatus,null)
                .set(RosterBase::getEmergencyContactName,null)
                .set(RosterBase::getEmergencyContactTel,null)
                .set(RosterBase::getSpouseName,null)
                .set(RosterBase::getChildName,null)
                .set(RosterBase::getHeadshot,null)
                .eq(RosterBase::getId,rosterBaseDTO.getId());
        this.update(wrapper);
        return rosterBaseDTO;
    }

    /**
     * 查询当前花名册推送子系统是否成功
     * @param rosterId
     * @return
     */
    public Map<String,Object> checkIsSyncFailed(String rosterId) {
        User user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getRosterId, rosterId));
        // 查询当前用户推送子系统是否成功， isPush != 1说明未推送成功
        List<SysUserFailedLog> sysUserFailedLogList = sysUserFailedLogService.list(new LambdaQueryWrapper<SysUserFailedLog>()
                .eq(SysUserFailedLog::getUserId, user.getId())
                .ne(SysUserFailedLog::getIsPush, "1")
        );
        Map<String,Object> res = new HashMap<>();
        if (sysUserFailedLogList.size() > 0) {
            res.put("success", false);
            List<String> msg = changeFailedLogToMsg(sysUserFailedLogList);
            if (msg.size() > 0) {
                String message = msg.stream().collect(Collectors.joining(","));
                message = message + "，请重新推送后再进行尝试";
                res.put("message", message);
            } else {
                res.put("message", "有推送失败的数据，请重新推送后再进行尝试");
            }
        } else {
            res.put("success", true);
        }
        return res;
    }

    /**
     * 查询推送失败的数据
     * @param rosterId
     * @return
     */
    public Map<String,Object> getPushFailedMsg(String rosterId) {
        User user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getRosterId, rosterId));
        // 查询当前用户推送子系统是否成功， isPush != 1说明未推送成功
        List<SysUserFailedLog> sysUserFailedLogList = sysUserFailedLogService.list(new LambdaQueryWrapper<SysUserFailedLog>()
                .eq(SysUserFailedLog::getUserId, user.getId())
                .ne(SysUserFailedLog::getIsPush, "1")
        );
        Map<String,Object> res = new HashMap<>();
        if (sysUserFailedLogList.size() > 0) {
            res.put("success", false);
            List<String> msg = changeFailedLogToMsg(sysUserFailedLogList);
            if (msg.size() > 0) {
                String message = msg.stream().collect(Collectors.joining(","));
                message = message + "，是否重新推送";
                res.put("message", message);
            } else {
                res.put("message", "有推送失败的数据，是否重新推送");
            }
        } else {
            res.put("success", true);
        }
        return res;
    }

    // 推送到子系统未成功的日志转换为msg
    public List<String> changeFailedLogToMsg(List<SysUserFailedLog> sysUserFailedLogList) {
        List<String> msg = new ArrayList<>();
        sysUserFailedLogList.stream().forEach(item -> {
            ServiceAliasEnum serviceAliasEnum = ServiceAliasEnum.getByValue(item.getBelongService());
            String label = serviceAliasEnum.getLabel();
            if ("0".equals(item.getIsSuccess())) {
                msg.add("“" + label + "”系统同步保存数据未成功");
            } else if ("1".equals(item.getIsSuccess())) {
                msg.add("清除”" + label + "“系统的数据未成功");
            } else if ("2".equals(item.getIsSuccess())) {
                msg.add("“" + label + "”系统同步删除数据未成功");
            }
        });
        return msg;
    }

    /**
     * 手动向子系统推送之前同步失败的数据
     * @param rosterId
     * @return
     */
    public boolean pushRosterSync(String rosterId) {
        AtomicBoolean respBool = new AtomicBoolean(true);
        RosterBase rosterBase = this.getById(rosterId);
        User user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getRosterId, rosterId));
        rosterBase.setUserId(user.getId());
        Map<String,Object> bodyMap = JSONObject.parseObject(JSON.toJSONString(rosterBase), Map.class);
        // 查询日志表中未重新推送的”删除”推送
        List<SysUserFailedLog> deleteUserFailedLogList = sysUserFailedLogService.list(new LambdaQueryWrapper<SysUserFailedLog>()
                .eq(SysUserFailedLog::getUserId, user.getId())
                .eq(SysUserFailedLog::getIsSuccess, "2") // 查询删除操作
                .ne(SysUserFailedLog::getIsPush, "1") // isPush != 1说明未推送成功
                .orderByDesc(SysUserFailedLog::getCreateDate)
        );
        // 查询当前用户推送子系统是否成功， isPush != 1说明未推送成功
        List<SysUserFailedLog> sysUserFailedLogList = sysUserFailedLogService.list(new LambdaQueryWrapper<SysUserFailedLog>()
                .eq(SysUserFailedLog::getUserId, user.getId())
                .ne(SysUserFailedLog::getIsPush, "1")
                .orderByDesc(SysUserFailedLog::getCreateDate)
        );
        if (CollectionUtil.isNotEmpty(sysUserFailedLogList)) {
            sysUserFailedLogList.stream().forEach(sysUserFailedLog -> {
                switch (sysUserFailedLog.getIsSuccess()) {
                    case "0": // 保存操作失败
                        boolean succ = false;
                        Object res = null;
                        if (ServiceAliasEnum.CPA_EVALUATION.getValue().equals(sysUserFailedLog.getBelongService())) {
                            // 调用评估系统的用户保存操作
                            res = restTemplateService.postCPA_EVALUATION("/roster/base/saveOrUpdateUserRoster",null,null,bodyMap);

                        } else if (ServiceAliasEnum.CPA_FINANCE.getValue().equals(sysUserFailedLog.getBelongService())) {
                            // 调用财务系统的用户保存操作
                            res = restTemplateService.postCPA_FINANCE("/roster/base/saveOrUpdateUserRoster",null,null,bodyMap);

                        } else if (ServiceAliasEnum.CPA_AUDIT.getValue().equals(sysUserFailedLog.getBelongService())) {
                            // 调用财务系统的用户保存操作
                            res = restTemplateService.postCPA_AUDIT("/roster/base/saveOrUpdateUserRoster",null,null,bodyMap);

                        }
                        if (ObjectUtil.isNotEmpty(res)) {
                            Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res), Map.class);
                            if ((Boolean) resMap.get("success")) {
                                // 保存成功，将推送日志表中的数据修改为推送完成
                                sysUserFailedLogService.finishById(sysUserFailedLog.getId());
                                succ = true;
                            }
                        }
                        if (!succ) { // succ的值为false，说明此次手动推送失败
                            respBool.set(false);
                        }
                        break;
                    case "1": case "2": // 1:清除操作失败（这里指的是切换所属系统，要清除原系统中的人员信息）   2:删除操作失败
                        boolean succ1 = false;
                        Object res1 = null;
                        if (ServiceAliasEnum.CPA_EVALUATION.getValue().equals(sysUserFailedLog.getBelongService())) {
                            // 调用评估系统的用户清除操作
                            res1 = restTemplateService.postCPA_EVALUATION("/roster/base/deleteUserRoster",null,null,bodyMap);
                        } else if (ServiceAliasEnum.CPA_FINANCE.getValue().equals(sysUserFailedLog.getBelongService())) {
                            // 调用财务系统的用户清除操作
                            res1 = restTemplateService.postCPA_FINANCE("/roster/base/deleteUserRoster",null,null,bodyMap);
                        } else if (ServiceAliasEnum.CPA_AUDIT.getValue().equals(sysUserFailedLog.getBelongService())) {
                            // 调用财务系统的用户清除操作
                            res1 = restTemplateService.postCPA_AUDIT("/roster/base/deleteUserRoster",null,null,bodyMap);
                        }
                        if (ObjectUtil.isNotEmpty(res1)) {
                            Map<String,Object> resMap = JSONObject.parseObject(JSON.toJSONString(res1), Map.class);
                            if ((Boolean) resMap.get("success")) {
                                // 清除成功，将推送日志表中的数据修改为推送完成
                                sysUserFailedLogService.finishById(sysUserFailedLog.getId());
                                succ1 = true;
                            }
                        }
                        if (!succ1) { // succ的值为false，说明此次手动推送失败
                            respBool.set(false);
                        }
                        break;
                }
            });
            // respBool值为true 说明推送失败的操作现已全部重新推送
            if (respBool.get()) {
                if (CollectionUtil.isNotEmpty(deleteUserFailedLogList)) { // deleteUserFailedLogList不是空，说明有删除操作
                    // 删除花名册数据
                    this.removeById(rosterId);
                    // 通过花名册id查询用户
                    User u = userService.getOne(new QueryWrapper<User>().lambda().eq(User::getRosterId, rosterId));
                    if (ObjectUtil.isNotEmpty(u)) {
                        // 删除用户
                        userService.removeById(u.getId());
                        UserDTO userDTO = new UserDTO();
                        userDTO.setId(u.getId());
                        // 删除用户关联的管理部门信息
                        userMapper.deleteUserManageOffice(userDTO);  //  根据用户id删除用户的管理部门信息
                        // 删除用户关联的岗位信息
                        userMapper.deleteUserPost(u.getId());
                        // 删除用户关联的角色信息
                        userMapper.deleteUserRole(u.getId());
                    }
                }
            }
        }
        return respBool.get();
    }
}
